/*
   global types used by the BACI interpreter

*/


#define  UNDEF_VALUE 0x3f3f   /* '??' as char, val of undef vbl */
#define  STEPMAX     4        /* max steps before process switch */
#define  STEPMAXP1   5

#define  STMAX    6000        /* size of stack */
#define  STKINCR  200         /* stack size for each process */

#define  MAXPRIO  0x7FFF      /* max priority a WAITC can have */

enum  pstat{   run,           /* normal operation */
               fin,           /* normal termination */
               divchk,        /* division error */
               inxchk,        /* array indexing error */
               stkchk,        /* stack addressing error */
               charchk,       /* invalid character */
               inpchk,        /* invalid input type */
               pcodchk,       /* invalid PCODE opcode */
               rdchk,         /* read error */
               wrtchk,        /* write error */
               padrchk,       /* invalid PCODE address */
               semchk,        /* invalid value in a general semaphore */
               bsemchk,       /* invalid value in a binary semaphore */
               uisemchk,      /* unititialized semaphore */
               xmonchk,       /* cross-monitor call */
               redchk,        /* attempt to read past EOF */
               procchk,       /* invalid process ID */
               proctabchk,    /* proctab full */
               deadlock       /* no runnable process */
         };

/* process table type -- information kept for each concurrent process */
typedef struct   {  
            unsigned    pc;         /* program counter */
            unsigned    t,b;        /* top, bottom of AR */
            int         tabix;      /* index into tab array */
            int         stacksize;  /* stack limit */
            int         display[LSIZE];   /* display */
            int         suspend;    /* -1 or stack addr of semaphore */
            int         active;     /* process active? */
            int         monitor;    /* 0 or tab index of monitor */
            int         priority;   /* WAITC priority P&W */
            int         atomic;     /* is process un-interruptable? */
            unsigned    xpc;        /* no. of pcode instrs executed */
            /* the next two fields are used to execute one source line */
            /* by the current process                                  */
            int         first_spc;  /* least pc for this source line */
            int         last_spc;   /* greatest pc for this source line */
   } proctab;

   /* values for the suspend field of the proctab struct */
#define  NOT_SUSPENDED  -1          /* proc is not suspended */
   /* when a normal proc suspends, the suspend field is the addr of */
   /* the semaphore or condition on which the proc is waiting       */
   /* The following constant is used when a proc suspends for a     */
   /* different reason, like proc[0] at a cobegin block or a proc   */
   /* suspended by the user through SUSPEND                         */
   /* Because of the algorithm in chooseproc, this number should be */
   /* positive and an invalid runtime stack address                 */
#define  GLOBAL_SUSPEND  STMAX
   /* This value is used when a monitor proc is suspended because   */
   /* it signals a condition (the Immediate Resumption Requirement) */
   /* Because of the algorithm in chooseproc, this number should be */
   /* positive and an invalid runtime stack address                 */
#define  SIGNAL_SUSPEND  STMAX+1

/*
 * $Log: bainterp.h,v $
 * Revision 1.4  2002/05/07 18:18:49  bynum
 * add SIGNAL_SUSPEND and change GLOBAL_SUSPEND to a positive value
 *
 * Revision 1.3  2000/06/07 17:49:58  bynum
 * incorporate code from BACI GUI directory
 *
 * Revision 1.4  1998/08/18 14:11:31  bynum
 * add first_spc, last_spc fields to proctab struct
 *
 * Revision 1.3  1998/06/08 10:18:05  bynum
 * add NON_SUSPENDED, GLOBAL_SUSPEND constants
 *
 * Revision 1.2  1997/12/16 19:43:16  bynum
 * rename MAXINT to MAXPRIO and reduce the value to 0x7FFF
 *
 * Revision 1.1  1997/07/02 13:33:34  bynum
 * Initial revision
 *
 */
